clc
clear all

addpath(genpath('Data'))
addpath(genpath('library'))

%% Load data

% Country list
countries={'AT','BE','DE','ES','FI', 'FR',  'IT', 'NL', 'PT','GR'}   

% EA Monetary policy shock
MP_data = readmatrix('EA_MP_Shock');

for i=1:length(countries)
 
% Set lag structure
P = 1; % VAR lags
Q = 2; % Narrative shock lags (Q+1 in regression)

% Load the data: 
% Order of variables: real GDP,  HICP, employment, wages, stock prices, and Gini market coefficients (all in log-levels)

data = readmatrix(countries{i});
data0=data;

%Load the monetary policy shock for the EA
exshock=MP_data(:,2);
exshock=exshock/std(exshock);   % 1 standard deviation shock
rb=1;

% Time dimension
timedata=[1999.25:0.25:2014.75];
tt1=find(timedata==1999.25);
tt2=find(timedata==2014.75);
data0=data0(tt1:tt2,:);
exshock=exshock(tt1:tt2,:);
data=[data0]; 

L=P; %lag length of the VAR
LS=Q; %Shock lag 
Y=data(:,1:end);
%take lags
X=[];
for j=1:L
X=[X lag0(Y,j) ];
end
X=[X ones(rows(X),1)];
Y=Y(L+1:end,:);
X=X(L+1:end,:);
T=rows(X);

N=cols(Y);

% Shock 
shock=exshock;
shock_lag=[];

for j=1:LS
shock_lag=[shock_lag lag0(shock,j) ];
end
shock=shock(L+1:end,:);
shock_lag=shock_lag(L+1:end,:);
% Endegenous
y=Y(LS:end,:);
% Exogenous
xvar=[X shock shock_lag];
xvar=xvar(LS:end,:);

EX=1+1+LS;
ncrit=(N*L+EX);


% OLS Estimation
b1=xvar\y;
st1=stability(b1,N,L,EX);  % If the st1==1 ==> not stable => change lags
r1=y-xvar*b1;
s1=r1'*r1/(size(r1,1)-size(X,2));   
sigma01=s1;

%/////////////////Priors and starting values///////////////////////
%Minnesota type prior for the VAR coefficients via dummy observations
[yd,xd,BETA0,SIGMA0]=getdummiesSS_v2(0.2,0,1e-5,Y,L,EX,0,0,1,Q,10);

jgibbs=1;
igibbs=1;
maxdraws=1000;
REPS=50000;
BURN=0.8*REPS;

for igibbs =1:REPS

igibbs;

[beta21,res1,sigma1,problem1,numtrys1]=getvarcoef_noSV(y,xvar,yd,xd,maxdraws,L,EX,sigma01);
if problem1
    beta21=varcoef01;
    sigma1=sigma01;
else
    varcoef01=beta21;
    sigma01=sigma1;
    
end



if igibbs>BURN

if problem1==0
bsave1(jgibbs,:,i)=beta21;
jgibbs=jgibbs+1;
end
end

end
end

beta_all=0;
for kkk=1:size(bsave1,3);
beta_all=beta_all+bsave1(:,:,kkk);
end


bsave1=beta_all/length(countries);

%% Doing Impulse Response Analysis
vars={'GDP', 'HICP', 'Employment' ,  'Wages', 'Stock prices', 'GINI'};

signshock=-rb*100;
hoz=20;
vars;

for kk=1:size(bsave1,1);  
beta=bsave1(kk,:,:)';  
yhat=zeros(hoz,N);
bparam0=reshape(beta,N*L+EX,N);
bparam=bparam0(1:N*L,:);
bshock=bparam0(N*L+2:end,:)*signshock;
LM=max(L,LS);        
for j=L+1:hoz
    Xl=[];    
    for kkk=1:L
    Xl=[Xl yhat(j-kkk,:)];
    end   
    if j< (L+1+LS)+1       
    yhat(j,:)=Xl*bparam+bshock(j-L,:);     
    else    
    yhat(j,:)=[Xl]*bparam;     
    end          
end
out(kk,:,:)=yhat;
end







 %% Plot figures
lb=16;
uu=84;
vars=vars(1:end);
tt=length(vars);
figure(5)
irfhorizon=10;
for i=1:size(data,2)
temp=out(:,:,i);    
temp1=squeeze(prctile(temp,[50 lb uu],1))';
temp2= temp1(L+1:L+irfhorizon,:);
IRFLB68=   temp2(:,2);
IRFUB68=temp2(:,3);
impulse1=temp2(:,1);

subplot(2,3,i)
x = 1:irfhorizon;
zeroline = zeros(irfhorizon,1);
plot(IRFLB68,'LineStyle',':','Color',[0 0 0],'LineWidth',1.5);
hold on
plot(IRFUB68,'LineStyle',':','Color',[0 0 0],'LineWidth',1.5);
hold on;
ha=shadedplot(x, IRFUB68', IRFLB68', [0.7 0.7 0.7],[0.7 0.7 0.7]);
hold on;
plot(impulse1,'-','Color',[0 0 0],'LineWidth',2);
ylabel('Percent');
hold on;
plot(zeroline(1:irfhorizon),':k')
title(vars(i))
axis tight
end

