% 

function [B,Q,sg]=INIW_v2(Y,K,N,L,lam1,lam3,lam4,conpos,inst)
% Y: Data
% K: Numbers of coefficients in each equation
% N: Numbers of variables
% L: Numbers of lags
% lam1: tightness prior in the AR(1)
% lam3: tightness of prior of higher order
% lam4: tightness of prior of the constant 
% conpos: =1 Intercept is 1st; 2= Interpcet is last ranked
% inst: Number of exogenous surprises 

% OUTPUT
% Priors: mean of coeffiences
% Priors variance of coeffiences
% sg: Sigma 

for i=1:N
    ytemp=Y(:,i);
    xtemp=[];
    for lag=1:L
    xtemp=[xtemp lag0(ytemp,lag)];
    end
    xtemp=[xtemp ones(rows(ytemp),1)];
    ytemp=ytemp(L+1:end,:);
    xtemp=xtemp(L+1:end,:);
    btemp=xtemp\ytemp;
    etemp=ytemp-xtemp*btemp;
    stemp=etemp'*etemp/rows(ytemp);
    if btemp(1)>1;
        btemp(1)=1;
    end
    sigmaP(i)=sqrt(stemp);
    bsave(i)=btemp(1);
end

sg=sigmaP;

Q=zeros(K,N);

% SETTING PRIORS

prior.minnesota.exog_std = lam4';  % For the constant term
prior.minnesota.decay = lam3;
prior.minnesota.tightness=lam1;
prior.minnesota.sigma=sg;
P=L;

temp1 = kron(((1:P)').^-prior.minnesota.decay, ones(N,N)); % p^-d  % Decaying Parameters
temp2 = repmat(prior.minnesota.sigma,P*N,1); % sigma_i
temp3 = repmat(prior.minnesota.sigma'.^-1,P,N); % /sigma_j
Q0 = prior.minnesota.tightness*temp1.*temp2.*temp3;

if conpos==2 % Intercept is at the end
Q0 = [Q0; repmat(prior.minnesota.exog_std,1,N)]; % this assumes there is only constant term in w
Q0(:,1:inst)=1e-10;
else
    
Q0 = [repmat(prior.minnesota.exog_std,1,N); Q0]; % this assumes there is only constant term in w
Q0(:,1:inst)=1e-10;    
end
Q=Q0.^2;


% The results are similar as below
% Q=zeros(K,N);
% for in=1:inst
% Q(1:K,in)=1e-10;  % Strict restriction for the first 
% end
% N1=N-inst;
% 
% for k=inst+1:N
%     for p=1:L
%         for v=1:N
%     Q((p-1)*N+v,k)=(lam1*sg(k)/sg(v)*(p^(-lam3)))^2;
%         end
%        
%     end
%     Q(N*L+1,k)=(lam4)^2;  % constant is ranked last 
%   
% end



B=zeros(K,N);
% B(1:N,1:N)=diag(nonsta);
% B(1:N,1:N)=diag(bsave);
for j=inst+2:N
      B(j,j)=bsave(j);
end

%  if SW==1
%      for j=inst+1:N
%      B(j,j)=1;
%      end
%  end

B=vec(B);
Q=diag(vec(Q));