function [h,w] = pgas_A(y, H, LH, logmu0, xdata,iamat,BETAM,sdet,sbeta,iQ,cQ,M,L)
% Conditional particle filter with ancestor sampling
% Input:
%   y - measurements
%   q - process noise variance
%   r - measurement noise variance
%   N - number of particles
%   X - conditioned particles - if not provided, un unconditional PF is run
%   NS- Number of state variables
%   logmu0- column vector
%   BETAM: Coefficients of VAR model (N*L + N*(LH+1) + Exo*(LX+1)+1)* N

N=size(y,2);
NS=size(H,1);
Nx=NS/(LH+1);
T = length(y);
h = zeros(NS, M, T); % States x Particles x times
a = zeros(M, T); % Ancestor indices
w = zeros(M, T); % Weights
h(:,1:M-1,1) = repmat(logmu0,1,M-1); % Deterministic initial condition
h(:,M,1) = H(:,1); % Set the 1st particle according to the conditioning


%randn('seed',123);
%rand('seed',123);

for t = 1:T
    if(t ~= 1)
        ind = discretesample(w(:,t-1),M);
        ind = ind(randperm(M));
        %ind=resampleSystematic(w(:,t-1),M);  % Using systematic resampling
        
        xpred=sbeta*h(:,:, t-1)+sdet(:,t);
        res=H(:,t)-xpred;
        m=zeros(M,1);
        for j=1:M
        r1=res(:,j)';
        m(j)=exp(-0.5*r1*iQ*r1');
        end
        
               
        h(:,:,t) = xpred(:,ind) + cQ*randn(NS,M);  % 
        %x(:,t) = xpred(ind) + sqrt(q)*randn(M,1);
        
            %x(M,t) = X(t); % Set the M:th particle according to the conditioning
            h(:,M,t) = H(:,t); % Set the M:th particle according to the conditioning
            % Ancestor sampling
            %m = exp(-1/(2*q)*(H(:,t)-xpred).^2);
            w_as = w(:,t-1).*m;
            w_as = w_as/sum(w_as);
            %ind(M) = find(rand(1) < cumsum(w_as),1,'first');
            ind(M) = discretesample(w_as,1);
        
        % Store the ancestor indices
        a(:,t) = ind;
    end
    % Compute importance weights
for k=1:M  
    state1= h(:,k,t)';  % Make it a row vector
    r=iamat*diag(exp(state1(1:Nx)))*iamat';
    detR=det(r);
    iR=inv(r);
    XX=[xdata(t,1:N*L) state1 xdata(t,N*L+1:end)]; 
    ypred=XX*BETAM;  
    res=y(t,:) - ypred;
    res2=res*iR*res';
    logweights(k) = log((1/sqrt(detR)))+(-0.5*res2); % (up to an additive constant)    
end
    const = max(logweights); % Subtract the maximum value for numerical stability
    weights = exp(logweights-const);
    w(:,t) = weights/sum(weights); % Save the normalized weights
end

% Generate the trajectories from ancestor indices
ind = a(:,T);
for t = T-1:-1:1
    h(:,:,t) = h(:,ind,t);
    ind = a(ind,t);
end
end

% function [St,m] = state_equation(St1,betas,Sfit,Sp,iQ,M)
% St=betas*St1+Sfit;
% res=Sp-St;
% m=zeros(1,M);
% for j=1:M
% r1=res(:,j)';
% m(j)=exp(-0.5*r1*iQ*r1');
% end
% end
